#___________________________________________________________________
#
#	citation :
#	Dechaume-Moncharmont, F.-X. et al. (2011) 
#	Sexing birds using discriminant function analysis: a critical appraisal. 
#	The Auk, 128(1):78-86. https://doi.org/10.1525/auk.2011.10129
#
#  	Author: fx dechaume-moncharmont (www.fxdm.org)
#	E-mail: fx.dechaume@univ-lyon1.fr
#  	Language: R version 4.2
#  	Last update: December, 10th 2022
#___________________________________________________________________
#

# indiv --> individual number
# ring_code --> individual ring code
# sex --> sex: male (M) or female (F)
# age --> age: Adult (A), Subadult, ou Junevile (J). The paper is based on adult dataset
# bill_length1 --> bill length (first measure)
# bill_length2 --> bill length (second measure)
# bill_width1 --> bill width (first measure)
# bill_width2 --> bill width (second measure)
# bill_depth1 --> bill depth (first measure)
# bill_depth2 --> bill depth (second measure)
# tarsus_right1 --> right tarsus length (first measure)
# tarsus_right2 --> right tarsus length (second measure)
# tarsus_left1 --> left tarsus length (first measure)
# tarsus_left2 --> left tarsus length (second measure)
# wing_right1 --> right wing chord (first measure)
# wing_right2 --> right wing chord (second measure)
# wing_left1 --> left wing chord (first measure)
# wing_left2 --> left wing chord (second measure)
# tail_length1 --> tail length (first measure)
# tail_length2 --> tail length (second measure)
# mass --> body mass
# head1 --> head plus bill length (first measure)
# head2 --> head plus bill length (first measure)

rm(list = ls())

####################################################################
###
###  Packages
###
####################################################################

# install.packages('MASS')
library(MASS)

# install.packages('boot')
library(boot)

####################################################################
###
###  uploading the dataset in R
###
####################################################################

# load of the original full dataset
complet <- read.table ( "dataset.dat" , header = T); 
# creation of a simplifier dataset with the adult only
temp <- na.omit(complet[complet$age == "A",])
write.table(temp , file ="adultep.dat")
adultep <- read.table ("adultep.dat" , header = T); 
rm(temp)
# computation of the mean value for each parameter
adultep$wing_left = (adultep$wing_left1+adultep$wing_left2)/2
adultep$wing_right = (adultep$wing_right1+adultep$wing_right2)/2
adultep$tarsus_left = (adultep$tarsus_left1+adultep$tarsus_left2)/2
adultep$tarsus_right = (adultep$tarsus_right1+adultep$tarsus_right2)/2
adultep$tail_length = (adultep$tail_length1+adultep$tail_length2)/2
adultep$head = (adultep$head1+adultep$head2)/2
adultep$bill_length = (adultep$bill_length1+adultep$bill_length2)/2
adultep$bill_width = (adultep$bill_width1+adultep$bill_width2)/2
adultep$bill_depth = (adultep$bill_depth1+adultep$bill_depth2)/2
adultep$tarsus = (adultep$tarsus_left + adultep$tarsus_right)/2
adultep$wing = (adultep$wing_right + adultep$wing_left)/2
summary(adultep)
length(adultep$indiv)
length(adultep$sex[adultep$sex=="M"])
length(adultep$sex[adultep$sex=="F"])

####################################################################
###
### 	simple DFA with the original dove dataset
###			method = LOOV = Jack-knife
###
####################################################################

fit <- qda(sex ~ wing + tarsus +
			+ bill_length + bill_width + bill_depth+ 
			+ mass  + tail_length + head,
			data = adultep, na.action="na.omit", CV=TRUE)
ct <- table(adultep$sex, fit$class)
length(adultep$sex[adultep$sex=="M"])
length(adultep$sex[adultep$sex=="F"])
### crossvalidation table
ct
### discriminant rate:
sum(diag(prop.table(ct)))

###
### Calculation of the bootstrapped 95% CI
###

nbsimul <- 1000
precision <- numeric(nbsimul)
for (i in 1:nbsimul) {
  newdataset <- adultep[sample(1:length(adultep$indiv), replace = T),]
  # summary(newdataset)
  fit <- qda(sex ~ wing + tarsus + 
               + bill_length + bill_width + bill_depth + 
               + mass  + tail_length + head,
             data = newdataset, na.action="na.omit", CV=T)
  length(newdataset$sex[newdataset$sex=="M"])
  length(newdataset$sex[newdataset$sex=="F"])
  ct <- table(newdataset$sex, fit$class)
  precision[i] <- sum(diag(prop.table(ct)))
  cat(round(100*i/nbsimul), "%\n")
}
hist(precision)
abline(v = mean(precision), col="red")
abline(v = quantile(precision, c(0.025, 0.975)))
quantile(precision, c(0.025, 0.975))

###
### Calculation of the bootstrapped 95% CI with package 'boot'
### 

scordfa <- function(D,d){
  E = D[d,]
  fit <- qda(sex ~ wing + tarsus + bill_length + bill_width + bill_depth+ mass  + tail_length + head,
             data = E, na.action="na.omit",  CV=T)
  ct <- table(E$sex, fit$class)
  return(sum(diag(prop.table(ct))))
}
scordfa(adultep)
results <- boot(data = adultep, statistic=scordfa, R = 500)
boot.ci(results, type="basic")

####################################################################
###
###  Simulation of the effect of sample size
###
####################################################################

effettaille <- NULL
effettaille$coeff <-NULL
effettaille$moy <- NULL
effettaille$sd <- NULL
effettaille$sem <- NULL
effettaille$n <- NULL
detail <- NULL
detail$samplesize <- NULL
detail$precision <- NULL
sample_n <- 40 # initial sample size
sample_size_step <- 10 # increasing step for the sample size
replicate_number <- 50
while(sample_n <= length(adultep$age)){
	print(sample_n/length(adultep$age)) # console output for control
	ratioprecision <- NULL
	precision <- NULL
	for (i in 1:replicate_number){
		echantill <- sample(1:length(adultep$sex), size = sample_n, replace = FALSE)
		echantillon <- adultep[echantill,]
		while((sum(echantillon$sex == "M")<10)||(sum(echantillon$sex == "F")<10))
		{
			echantill <- sample(1:length(adultep$sex), size = sample_n, replace = FALSE)
			echantillon <- adultep[echantill,]
		}
		fit <- qda (sex ~ wing_left + wing_right + tarsus_right + tarsus_left + 
			+ bill_length + bill_width + bill_depth+ 
			+ mass  + tail_length + head,
			data = echantillon, na.action="na.omit", CV=TRUE)
		ct <- table(echantillon$sex, fit$class)
		precision <- c(precision, sum(diag(prop.table(ct))))
		detail$precision <- c(detail$precision, sum(diag(prop.table(ct))))
		detail$samplesize <- c(detail$samplesize, sample_n)
	}
	effettaille$samplesize <- c(effettaille$samplesize, sample_n)
	effettaille$moy <- c(effettaille$moy, mean(precision))
	effettaille$sd <- c(effettaille$sd, sd(precision))
	effettaille$sem <- c(effettaille$sem, sd(precision)/sqrt(length(precision)))
	effettaille$n <- c(effettaille$n ,length(precision))
	sample_n <- sample_n + sample_size_step
}

####
#### save and load simulation data
####
write.table(effettaille, file ="effettaille02.dat")
effettaille <- read.table("effettaille02.dat", header=T)
summary(effettaille)
write.table(detail, file ="detailtaille02.dat")
detail <- read.table("detailtaille02.dat", header=T)
summary(detail)

####
#### output plot
####
plot(detail$samplesize, detail$precision + rnorm(length(detail$precision),mean=0,sd=0.005), 
	main=NULL,
	xlab = "Sample size",
	ylim = c(0.4,0.9),
	ylab = "Discriminant rate",
	col = "grey")
lines(smooth.spline(effettaille$samplesize , effettaille$moy, df = 19), col = "red", lwd = 4)